#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
##############################################################################
#
# harmony.py
# Creates a table for the words with a count of "unharmonical" glyphs.
#
# See http://voynich.tamagothi.de/2008/07/05/die-harmonie-der-glyphenfolgen/
# for a description of the harmonic rules in the VMS text in german language.
#
# PROBLEMS:
#
# There is a little MySQL syntax in the queries. This may harden a port
# to another RDBMS.
#
# AUTHOR:
#
# Written by Elias Schwerdtfeger
# http://www.tamagothi.de/
#
# $Id: harmony.py,v 1.1 2008/08/22 19:21:41 elias Exp $
#
# $Log: harmony.py,v $
# Revision 1.1  2008/08/22 19:21:41  elias
# Initial revision
#
##############################################################################


import MySQLdb
import re

# Change the following stmt to match your database
# My own settings are far from being secure ones... ;-)

db = MySQLdb.connect(db='voynich', user='voynich', passwd='voynich')

# Regular expressions assigning the glyphs to chracter classes

ra = []

# iGh - changes from i to e: State J
ra.append(('JE', re.compile(r'i[fpkt]?h')))

# cGh - e-sequence: State E
ra.append(('E', re.compile(r'c[fpkt]?h')))

# Gallow: State E
ra.append(('E', re.compile(r'[fpkt]')))

# a - change from e to i: State A
ra.append(('A', re.compile(r'a')))

# q - only at the beginning of a word, change to E: State Q
ra.append(('QE', re.compile(r'q')))

# e glyphs: State E
ra.append(('E', re.compile(r'[cheosgdy]')))

# i glyphs: State I
ra.append(('I', re.compile(r'[inrm]')))

# l joker glyph: State L
ra.append(('L', re.compile(r'l')))

# All remaining small caps in a word are weirdos: State W
ra.append(('W', re.compile(r'[a-z]')))


# Count the "unharmonical" glyphs

def harmonize(w):
    for (s, r) in ra:
        w = r.sub(s, w)
    # Initial is the "joker" state...
    state = 'L'
    count = 0
    wlen = len(w)
    lst = wlen - 1
    for i in xrange(wlen):
        ac = w[i]
        if state == 'E':
            # In state E:
            # E, L or A are allowed except for last glyph in word
            if ac not in 'ELA':
                if i != lst:
                    count += 1
        elif state == 'I':
            # In state I:
            # I, L or J are allowed except for last glyph in word
            if ac not in 'ILJ':
                if i != lst:
                    count += 1
        elif state == 'L':
            # In state L:
            # Everything than a Q is allowed, except for first glyph in word
            if ac in 'Q':
                if i != 0:
                    count += 1
        elif state == 'Q':
            # In state Q:
            # E or A are allowed
            if ac not in 'EA':
                count += 1
        elif state == 'A':
            # In state A:
            # I or J are allowed, except for the last glyph in word
            if ac not in 'IJ':
                if i != lst:
                    count += 1
        elif state == 'J':
            # In state J:
            # Only E is allowed
            if ac not in 'E':
                count += 1
        elif state == 'W':
            # In state W (weirdo):
            # Everything is wrong, except for the last glyph in word
            if i != lst:
                count += 1
        else:
            # That's a line of debugging code, helping me to identify
            # improper states
            raise RuntimeError(ow + ' ' + w + ' ' + state)
        state = ac
    return count;
    

cur = db.cursor()
cur.execute("""
CREATE TABLE IF NOT EXISTS voy_harmony
(
harm_word SMALLINT(6) PRIMARY KEY,
harm_failcount TINYINT NOT NULL
)
""")
cur.execute("""
DELETE FROM voy_harmony
""")

enum = db.cursor()
enum.execute("""
SELECT word_id, eva
FROM voy_word
WHERE readable = 'Y'
""")

for (id, eva) in enum.fetchall():
    cur.execute("""
    INSERT INTO voy_harmony
    ( harm_word, harm_failcount )
    VALUES
    ( %s, %s )
    """, (id, harmonize(eva)))
enum.close()
cur.close()
